﻿Imports [Lib]
Imports DTEBoxCliente

''' <summary>
''' Formulario para mostrar el XML generado a partir del objeto Factura y enviar la Factura al DTEBox.
''' </summary>
''' <remarks></remarks>
Public Class FormSalvar

    ''' <summary>
    ''' Dirección ip o nombre del servidor del DTEBox.
    ''' </summary>
    ''' <remarks></remarks>
    Public IpDTEBox As String

    ''' <summary>
    ''' Llave de Autenticación para la conección al DTEBox.
    ''' </summary>
    ''' <remarks></remarks>
    Public LlaveAutenticacion As String

    ''' <summary>
    ''' Factura a enviar al DTEBox.
    ''' </summary>
    ''' <remarks></remarks>
    Public Factura As IFactura

    Public Sub New(ByRef factura As IFactura, ByVal ipDTEBox As String, ByVal llaveAutenticacion As String)

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me.Factura = factura
        Me.IpDTEBox = ipDTEBox
        Me.LlaveAutenticacion = llaveAutenticacion
        textXMLFactura.Text = Me.Factura.ObtenerXml()
    End Sub

    ''' <summary>
    ''' Manejador del evento click del botón Cancelar. Cierra el Formulario.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnCancelar_Click(sender As Object, e As EventArgs) Handles btnCancelar.Click
        Close()
    End Sub

    ''' <summary>
    ''' Manejador del evento click del botón Enviar al DTEBox. Aqúi se envía la Factura al DTEBox.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnSalvar_Click(sender As Object, e As EventArgs) Handles btnSalvar.Click
        Try
            'Llamada al servicio
            Dim ambiente As DTEBoxCliente.Ambiente
            ambiente = DTEBoxCliente.Ambiente.Homologacion
            Dim fechaResolucion As Date
            fechaResolucion = New DateTime(2012, 1, 1)
            Dim numeroResolucion As Integer
            numeroResolucion = 0
            Dim tipoPdf417 As DTEBoxCliente.TipoPdf417
            tipoPdf417 = DTEBoxCliente.TipoPdf417.Fuente

            Dim apiURL As String
            Dim apiAuth As String
            apiURL = "http://<DTEBox IP>/api/Core.svc/Core".Replace("<DTEBox IP>", IpDTEBox)
            apiAuth = LlaveAutenticacion

            Dim servicio As DTEBoxCliente.Servicio
            servicio = New DTEBoxCliente.Servicio(apiURL, apiAuth)

            Dim resultadoEnvioDte As DTEBoxCliente.ResultadoEnvioDte = Nothing
            Dim resultadoEnvioBoleta As DTEBoxCliente.ResultadoEnvioBoleta = Nothing
            Dim resultado As DTEBoxCliente.EstadoResultado = DTEBoxCliente.EstadoResultado.Ok
            Dim esDte = True

            If TypeOf Factura Is Exportaciones Then
                resultadoEnvioDte = servicio.EnviarDocumento(CType(Factura, Exportaciones).GetDTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417)
                resultado = resultadoEnvioDte.ResultadoServicio.Estado
                esDte = True
            ElseIf TypeOf Factura Is Documento Then
                resultadoEnvioDte = servicio.EnviarDocumento(CType(Factura, Documento).DTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417)
                resultado = resultadoEnvioDte.ResultadoServicio.Estado
                esDte = True
            ElseIf TypeOf Factura Is Liquidacion Then
                resultadoEnvioDte = servicio.EnviarDocumento(CType(Factura, Liquidacion).GetDTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417)
                resultado = resultadoEnvioDte.ResultadoServicio.Estado
                esDte = True
            ElseIf TypeOf Factura Is Boleta Then
                resultadoEnvioBoleta = servicio.EnviarDocumento(CType(Factura, Boleta).GetDTE, ambiente, fechaResolucion, numeroResolucion, tipoPdf417)
                resultado = resultadoEnvioBoleta.ResultadoServicio.Estado
                esDte = False
            End If

            If resultado = DTEBoxCliente.EstadoResultado.Ok Then
                MessageBox.Show("Factura enviada correctamente al DTEBox", "Información", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Else
                Dim description As String = ""
                If esDte Then
                    description = resultadoEnvioDte.ResultadoServicio.Descripcion
                Else
                    description = resultadoEnvioBoleta.ResultadoServicio.Descripcion
                End If
                MessageBox.Show(description, "Error enviado desde el DTEBox", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error enviando la Factura al DTEBox", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub
End Class