﻿Imports DocumentoDTE.SiiDte
Imports [Lib]

''' <summary>
''' Formulario para gestionar los documentos de tipo Boleta.
''' </summary>
''' <remarks></remarks>
Public Class LiqForm
    ''' <summary>
    ''' Lista para almacenar los códigos de la sección de detalles.
    ''' </summary>
    ''' <remarks></remarks>
    Private ListaCdgItem As List(Of CdgItemA)

    ''' <summary>
    ''' Liquidación.
    ''' </summary>
    ''' <remarks></remarks>
    Public Liquidacion As [Lib].Liquidacion

    ''' <summary>
    ''' Definición de las diferentes reglas de validación para los campos de la interfaz de usuario en este formulario.
    ''' </summary>
    ''' <remarks></remarks>
    Public definicionesValidacion As Dictionary(Of String, Validacion)

    ''' <summary>
    ''' Manejador del evento  load del Fomulario
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub LiqForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Me.Text = String.Format("Liquidación - {0:dd-MM-yyyy HH:mm:ss}", DateTime.Now)

        ListaCdgItem = New List(Of CdgItemA)
        UtilIU.LlenarValoresDesde(comboIndExe, GetType(Enumerations.DetalleA_IndExe))

        Dim doc As New List(Of String)(New String() {"43"})
        Dim listTipoDoc As List(Of KeyValuePair(Of String, String)) = AsistenteEnum.ListaTipoDocumento().FindAll(Function(d) doc.Contains(d.Key))
        UtilIU.LlenarValoresDesdeLista(comboTpoDocRef, listTipoDoc)
        UtilIU.LlenarValoresDesdeLista(comboTipoDTE, listTipoDoc)

        UtilIU.LlenarValoresDesdeLista(comboTpoDocLiq, AsistenteEnum.ListaTipoDocumento())

        Dim listUM As List(Of KeyValuePair(Of String, String)) = AsistenteEnum.ListaUnidadMedida()
        listUM.Insert(0, New KeyValuePair(Of String, String)("", ""))
        UtilIU.LlenarValoresDesdeLista(comboUnmdItem, listUM)

        Liquidacion = New [Lib].Liquidacion()

        ' Definiendo reglas de validacion
        definicionesValidacion = New Dictionary(Of String, Validacion) From {
            {"textRUTEmisor", New Validacion("RUT del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}, {"rut", True}})},
            {"textRznSoc", New Validacion("Razón social del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textGiroEmis", New Validacion("Giro del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textDirOrigen", New Validacion("Dirección del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textCmnaOrigen", New Validacion("Comuna del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textCiudadOrigen", New Validacion("Ciudad del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textActeco", New Validacion("Acteco del Emisor", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textRUTRecep", New Validacion("RUT del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}, {"rut", True}})},
            {"textRznSocRecep", New Validacion("Razón social del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textGiroRecep", New Validacion("Giro del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textDirRecep", New Validacion("Dirección del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textCmnaRecep", New Validacion("Comuna del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textCiudadRecep", New Validacion("Ciudad del Cliente", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textFolio", New Validacion("Folio", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}, {"num", True}})},
            {"textMntNeto", New Validacion("Monto Neto", "Encabezado", New Dictionary(Of String, Object) From {{"num", True}})},
            {"textIva", New Validacion("IVA", "Encabezado", New Dictionary(Of String, Object) From {{"num", True}})},
            {"textMntExe", New Validacion("IVA", "Monto Exento", New Dictionary(Of String, Object) From {{"num", True}})},
            {"textMntTotal", New Validacion("Monto Total", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}, {"num", True}})},
            {"textIpDteBox", New Validacion("IP DTEBOX", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textLlaveAutenticacion", New Validacion("Llave de Autenticación", "Encabezado", New Dictionary(Of String, Object) From {{"req", True}})},
            {"comboTpoDocLiq", New Validacion("Tipo de Documento", "Detalle", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textNmbItem", New Validacion("Nombre", "Detalle", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textQtyItem", New Validacion("Cantidad", "Detalle", New Dictionary(Of String, Object) From {{"req", True}, {"num", True}, {"min", New Decimal(0.000001)}})},
            {"textPrcItem", New Validacion("Precio U.", "Detalle", New Dictionary(Of String, Object) From {{"req", True}, {"num", True}, {"min", New Decimal(0.000001)}})},
            {"textMontoItem", New Validacion("Monto", "Detalle", New Dictionary(Of String, Object) From {{"req", True}, {"num", True}})},
            {"comboTpoDocRef", New Validacion("Tipo de documento", "Referencia", New Dictionary(Of String, Object) From {{"req", True}})},
            {"textFolioRef", New Validacion("Folio", "Referencia", New Dictionary(Of String, Object) From {{"req", True}})},
            {"dataGridDetalles", New Validacion("Detalle", "Detalles", New Dictionary(Of String, Object) From {{"req", True}})}
        }

    End Sub

    ''' <summary>
    ''' Limpia el formulario de entrada de Referencias
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LimpiarFormReferencia()
        comboTpoDocRef.SelectedIndex = 0
        textFolioRef.Text = ""
        dateFchRef.Text = ""
        textRazonRef.Text = ""
        textComentarioRef.Text = ""
    End Sub

    ''' <summary>
    ''' Limpia el formulario de entrada de Detalles
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LimpiarFormDetalle()
        textCdgItem.Text = ""
        textNmbItem.Text = ""
        textDscItem.Text = ""
        textQtyItem.Value = 0.0
        textPrcItem.Value = 0.0
        textMontoItem.Value = 0.0
        comboUnmdItem.SelectedIndex = 0
        chk_comboIndExe.Checked = False
        comboIndExe.SelectedIndex = 0
        comboTpoDocLiq.SelectedIndex = 0
    End Sub

    ''' <summary>
    ''' Adiciona o actualiza una referencia al grid de Referencias
    ''' </summary>
    ''' <param name="indice">indice de la referencia a actualizar, si es -1 se adiciona</param>
    ''' <remarks></remarks>
    Private Sub AdicionarOActualizarReferenciaEnTabla(ByVal indice As Integer)
        If indice < 0 Then
            dataGridReferencia.Rows.Add(CType(comboTpoDocRef.SelectedItem, InfoMostrar).Text, textFolioRef.Text, dateFchRef.Text, textRazonRef.Text, "")
        Else
            dataGridReferencia.Rows.Item(indice).SetValues(CType(comboTpoDocRef.SelectedItem, InfoMostrar).Text, textFolioRef.Text, dateFchRef.Text, textRazonRef.Text, "")
        End If

        LimpiarFormReferencia()
        dataGridReferencia.ClearSelection()

    End Sub

    ''' <summary>
    ''' Adiciona o actualiza un Detalle al grid de Detalles
    ''' </summary>
    ''' <param name="indice">indice del detalle a actualizar, si es -1 se adiciona</param>
    ''' <remarks></remarks>
    Private Sub AdicionarOActualizarDetalleEnTabla(ByVal indice As Integer)
        Dim textIndExe As String = ""
        If chk_comboIndExe.Checked Then
            textIndExe = CType(comboIndExe.SelectedItem, InfoMostrar).Text
        End If
        If indice < 0 Then
            dataGridDetalles.Rows.Add(textCdgItem.Text, CType(comboTpoDocLiq.SelectedItem, InfoMostrar).Text, textNmbItem.Text, textDscItem.Text, CType(comboUnmdItem.SelectedItem, InfoMostrar).Text,
                                      textQtyItem.Value.ToString(), textPrcItem.Value.ToString(), textMontoItem.Value.ToString(), textIndExe)
        Else
            dataGridDetalles.Rows.Item(indice).SetValues(textCdgItem.Text, CType(comboTpoDocLiq.SelectedItem, InfoMostrar).Text, textNmbItem.Text, textDscItem.Text, CType(comboUnmdItem.SelectedItem, InfoMostrar).Text,
                                      textQtyItem.Value.ToString(), textPrcItem.Value.ToString(), textMontoItem.Value.ToString(), textIndExe)
        End If

        LimpiarFormDetalle()
        dataGridDetalles.ClearSelection()

    End Sub

    ''' <summary>
    ''' Valida la sección de encabezado del Fomulario
    ''' </summary>
    ''' <returns>String con los mensajes de error en caso de que existan</returns>
    ''' <remarks></remarks>
    Private Function ValidarForm() As String
        ' validar la seccion de encabezado(emisor, cliente, totales)
        Dim mensajesError As String = UtilIU.ValidarFormulario(Me, New List(Of String) From {"Encabezado", "Detalles"}, definicionesValidacion)
        Return mensajesError
    End Function

    ''' <summary>
    ''' Valida la entrada de un Detalle
    ''' </summary>
    ''' <returns>String con los mensajes de error en caso de que existan</returns>
    ''' <remarks></remarks>
    Private Function ValidarFormDetalle() As String
        ' validar la seccion de Detalle
        Dim mensajesError As String = UtilIU.ValidarFormulario(Me, New List(Of String) From {"Detalle"}, definicionesValidacion)
        Return mensajesError
    End Function

    ''' <summary>
    ''' Valida la entrada de una Referencia
    ''' </summary>
    ''' <returns>String con los mensajes de error en caso de que existan</returns>
    ''' <remarks></remarks>
    Private Function ValidarFormReferencia() As String
        ' validar la seccion de Referencias
        Dim mensajesError As String = UtilIU.ValidarFormulario(Me, New List(Of String) From {"Referencia"}, definicionesValidacion)
        Return mensajesError
    End Function

    ''' <summary>
    ''' Adiciona o actualiza un Detalle
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridDetalles_BotonAdicionar_Click(sender As Object, e As EventArgs) Handles dataGridDetalles_BotonAdicionar.Click
        Try
            Dim mensajesError = ValidarFormDetalle()
            If String.IsNullOrEmpty(mensajesError) Then
                Dim indice As Integer = 0
                Dim valIndExe As String = ""
                If chk_comboIndExe.Checked Then
                    valIndExe = comboIndExe.SelectedValue
                End If
                Dim listaCdgItem As List(Of CdgItemA) = New List(Of CdgItemA)
                If Not textCdgItem.Text Is Nothing Then
                    listaCdgItem.Add(Liquidacion.CrearDetalleCdgItem("INT", textCdgItem.Text))
                End If
                Dim valUnmdItem As String = ""
                If comboUnmdItem.SelectedIndex <> 0 Then
                    valUnmdItem = comboUnmdItem.SelectedValue
                End If

                If dataGridDetalles.SelectedRows.Count > 0 Then
                    ' Editar
                    indice = dataGridDetalles.CurrentRow.Index
                    Liquidacion.ActualizarDetalleEnIndice(textNmbItem.Text, textMontoItem.Value, valIndExe, textDscItem.Text, textQtyItem.Value, valUnmdItem, textPrcItem.Value, listaCdgItem, comboTpoDocLiq.SelectedValue, indice)
                Else
                    ' Adicionar
                    indice = -1
                    Liquidacion.AdicionarDetalle(textNmbItem.Text, textMontoItem.Value, valIndExe, textDscItem.Text, textQtyItem.Value, valUnmdItem, textPrcItem.Value, listaCdgItem, comboTpoDocLiq.SelectedValue)
                End If

                AdicionarOActualizarDetalleEnTabla(indice)
            Else
                MessageBox.Show(mensajesError, "Errores de Validación", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Adiciona o actualiza una Referencia
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridReferencia_BotonAdicionar_Click(sender As Object, e As EventArgs) Handles dataGridReferencia_BotonAdicionar.Click
        Try
            Dim mensajesError = ValidarFormReferencia()
            If String.IsNullOrEmpty(mensajesError) Then
                Dim indice As Integer = 0
                If dataGridReferencia.SelectedRows.Count > 0 Then
                    ' Editar
                    indice = dataGridReferencia.CurrentRow.Index
                    Liquidacion.ActualizarReferenciaEnIndice(comboTpoDocRef.SelectedValue.ToString, textFolioRef.Text, dateFchRef.Value.Date, textRazonRef.Text, indice)
                Else
                    ' Adicionar
                    indice = -1
                    Liquidacion.AdicionarReferencia(comboTpoDocRef.SelectedValue.ToString, textFolioRef.Text, dateFchRef.Value.Date, textRazonRef.Text)
                End If
                AdicionarOActualizarReferenciaEnTabla(indice)
            Else
                MessageBox.Show(mensajesError, "Errores de Validación", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Llena el formulario de las referencias con los valores de la que se seleccione en el grid
    ''' </summary>
    ''' <param name="indice"></param>
    ''' <remarks></remarks>
    Private Sub LlenarFormReferencia(ByVal indice As Integer)
        Try
            Dim ref As ReferenciaA = Liquidacion.ListaReferencias.Item(indice)
            comboTpoDocRef.SelectedValue = ref.TpoDocRef
            textFolioRef.Text = ref.FolioRef
            dateFchRef.Value = ref.FchRef.AsDateTime.Date
            textRazonRef.Text = ref.RazonRef
            textComentarioRef.Text = ""
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Llena el formulario de los Detalles con los valores del que se seleccione en el grid
    ''' </summary>
    ''' <param name="indice"></param>
    ''' <remarks></remarks>
    Private Sub LlenarFormDetalle(ByVal indice As Integer)
        Try
            Dim det As DetalleA = Liquidacion.ListaDetalles.Item(indice)
            comboUnmdItem.SelectedValue = det.UnmdItem
            If det.IsValidIndExe Then
                chk_comboIndExe.Checked = True
                comboIndExe.SelectedValue = Enumerations.Detalle_IndExeToString(det.IndExe)
            Else
                chk_comboIndExe.Checked = False
                comboIndExe.SelectedIndex = 0
            End If
            If det.CdgItem.Count > 0 Then
                textCdgItem.Text = det.CdgItem.Item(0).VlrCodigo
            Else
                textCdgItem.Text = ""
            End If

            textNmbItem.Text = det.NmbItem
            textDscItem.Text = det.DscItem
            textQtyItem.Value = det.QtyItem
            textPrcItem.Value = det.PrcItem
            textMontoItem.Value = New Decimal(det.MontoItem.LongValue)
            comboTpoDocLiq.SelectedValue = det.TpoDocLiq
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Evento que se ejecuta al cambiar la selección de una referencia en el grid
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridReferencia_SelectionChanged(sender As Object, e As EventArgs) Handles dataGridReferencia.SelectionChanged
        Try
            If dataGridReferencia.SelectedRows.Count > 0 Then
                Dim indice As Integer = dataGridReferencia.CurrentRow.Index
                dataGridReferencia_BotonAdicionar.Text = "Actualizar"
                dataGridReferencia_BotonEliminar.Enabled = True

                LlenarFormReferencia(indice)

            Else
                dataGridReferencia_BotonEliminar.Enabled = False
                dataGridReferencia_BotonAdicionar.Text = "Adicionar"
                LimpiarFormReferencia()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Elimina la referencia seleccionada en el grid
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridReferencia_BotonEliminar_Click(sender As Object, e As EventArgs) Handles dataGridReferencia_BotonEliminar.Click
        Try
            If MessageBox.Show("Está seguro que desea eliminar?", "Eliminar", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                If dataGridReferencia.SelectedRows.Count > 0 Then
                    Dim indice As Integer = dataGridReferencia.CurrentRow.Index
                    dataGridReferencia.Rows.RemoveAt(indice)
                    Liquidacion.ListaReferencias.RemoveAt(indice)
                    dataGridReferencia.ClearSelection()
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Limpia el formulario de entrada de referencias
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridReferencia_BotonLimpiar_Click(sender As Object, e As EventArgs) Handles dataGridReferencia_BotonLimpiar.Click
        Try
            If dataGridReferencia.SelectedRows.Count > 0 Then
                dataGridReferencia.ClearSelection()
            Else
                LimpiarFormReferencia()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Evento que se ejecuta al cambiar el valor del campo Checked del CheckBox chk_comboIndExe
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub chk_comboIndExe_CheckedChanged(sender As Object, e As EventArgs) Handles chk_comboIndExe.CheckedChanged
        If chk_comboIndExe.Checked Then
            comboIndExe.Enabled = True
        Else
            comboIndExe.Enabled = False
            comboIndExe.SelectedIndex = 0
        End If
    End Sub

    ''' <summary>
    ''' Se ejecuta al cambiar los valores de los campos cantidad y precio para calcular el monto
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub textQtyItem_ValueChanged(sender As Object, e As EventArgs) Handles textQtyItem.ValueChanged, textPrcItem.ValueChanged
        textMontoItem.Value = textQtyItem.Value * textPrcItem.Value
    End Sub

    ''' <summary>
    ''' Evento que se ejecuta al cambiar la selección de una referencia en el grid
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridDetalles_SelectionChanged(sender As Object, e As EventArgs) Handles dataGridDetalles.SelectionChanged
        Try
            If dataGridDetalles.SelectedRows.Count > 0 Then
                Dim indice As Integer = dataGridDetalles.CurrentRow.Index
                dataGridDetalles_BotonAdicionar.Text = "Actualizar"
                dataGridDetalles_BotonEliminar.Enabled = True

                LlenarFormDetalle(indice)

            Else
                dataGridDetalles_BotonEliminar.Enabled = False
                dataGridDetalles_BotonAdicionar.Text = "Adicionar"
                LimpiarFormDetalle()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Elimina el Detalle seleccionado en el grid
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridDetalles_BotonEliminar_Click(sender As Object, e As EventArgs) Handles dataGridDetalles_BotonEliminar.Click
        Try
            If MessageBox.Show("Está seguro que desea eliminar?", "Eliminar", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                If dataGridDetalles.SelectedRows.Count > 0 Then
                    Dim indice As Integer = dataGridDetalles.CurrentRow.Index
                    dataGridDetalles.Rows.RemoveAt(indice)
                    Liquidacion.ListaDetalles.RemoveAt(indice)
                    dataGridDetalles.ClearSelection()
                End If
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Limpia el formulario de entrada de detalles
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dataGridDetalles_BotonLimpiar_Click(sender As Object, e As EventArgs) Handles dataGridDetalles_BotonLimpiar.Click
        Try
            If dataGridDetalles.SelectedRows.Count > 0 Then
                dataGridDetalles.ClearSelection()
            Else
                LimpiarFormDetalle()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Genera el XML con los valores entrados. El XML se muestra en un Dialogo.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnSalvar_Click(sender As Object, e As EventArgs) Handles btnSalvar.Click
        Try
            Dim mensajesError = ValidarForm()
            If String.IsNullOrEmpty(mensajesError) Then
                ' Setear Valores IdDoc
                Liquidacion.SetEncabezadoIdDoc(comboTipoDTE.SelectedValue, textFolio.Value, dateFchEmis.Value.Date)

                ' Setear valores Emisor
                Liquidacion.SetEncabezadoEmisor(textRUTEmisor.Text, textRznSoc.Text, textGiroEmis.Text, textActeco.Text, textDirOrigen.Text, textCmnaOrigen.Text, textCiudadOrigen.Text)

                ' Setear valores Receptor
                Liquidacion.SetEncabezadoReceptor(textRUTRecep.Text, textRznSocRecep.Text, textGiroRecep.Text, textDirRecep.Text, textCmnaRecep.Text, textCiudadRecep.Text)

                ' Setear valores totales
                Liquidacion.SetEncabezadoTotales(textMntTotal.Value, textMntExe.Value, textMntNeto.Value, textIva.Value)

                Dim formSalvar As FormSalvar = New FormSalvar(Liquidacion, textIpDteBox.Text, textLlaveAutenticacion.Text)
                formSalvar.ShowDialog()
            Else
                MessageBox.Show(mensajesError, "Errores de Validación", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ''' <summary>
    ''' Cierra el Fomulario
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnCerrar_Click(sender As Object, e As EventArgs) Handles btnCerrar.Click
        Close()
    End Sub
End Class