﻿Imports System.Collections.Specialized
Imports [Lib]
Imports System.Text

''' <summary>
''' Clase utilitaria para el trabajo con las interfaz de usuario.
''' </summary>
''' <remarks></remarks>
Module UtilIU

    ''' <summary>
    ''' Obtiene una lista de valores(InfoMostrar) a mostrar en los combo para los tipos Enuerados.
    ''' </summary>
    ''' <param name="typeEnum">Tipo del Enumerado a cargar en el combo</param>
    ''' <returns>Lista de Objetos de tipo InfoMostrar a cargar en los combo</returns>
    ''' <remarks></remarks>
    Function GetValoresMostrarPara(ByRef typeEnum As Type) As IList(Of InfoMostrar)
        Dim valores As List(Of InfoMostrar) = New List(Of InfoMostrar)
        Dim nombres As StringCollection = New StringCollection()
        If GetType(DocumentoDTE.SiiDte.Enumerations.DetalleB_IndExe) Is typeEnum Then
            nombres = DocumentoDTE.SiiDte.Enumerations.DetalleB_IndExeNames()
        ElseIf GetType(DocumentoDTE.SiiDte.Enumerations.Detalle_IndExe) Is typeEnum Then
            nombres = DocumentoDTE.SiiDte.Enumerations.Detalle_IndExeNames()
        ElseIf GetType(DocumentoDTE.SiiDte.Enumerations.DetalleA_IndExe) Is typeEnum Then
            nombres = DocumentoDTE.SiiDte.Enumerations.DetalleA_IndExeNames()
        ElseIf GetType(DocumentoBoleta.SiiDte.Enumerations.Detalle_IndExe) Is typeEnum Then
            nombres = DocumentoBoleta.SiiDte.Enumerations.Detalle_IndExeNames()
            nombres.Remove("6")
        End If

        For Each name As String In nombres
            Dim infoMostrar As InfoMostrar = New InfoMostrar()
            infoMostrar.Valor = name
            infoMostrar.Text = AsistenteEnum.ObtenerDescripcion(typeEnum, name)
            valores.Add(infoMostrar)
        Next

        Return valores
    End Function

    ''' <summary>
    ''' Obtiene una lista de valores(InfoMostrar) a mostrar en los combo desde una lista de elementos de tipo KeyValuePair(Of String, String).
    ''' </summary>
    ''' <param name="lista">Lista de objetos de tipo KeyValuePair(Of String, String)</param>
    ''' <returns>Lista de Objetos de tipo InfoMostrar a cargar en los combo</returns>
    ''' <remarks></remarks>
    Public Function GetValoresMostrarDesdeLista(ByRef lista As IList(Of KeyValuePair(Of String, String))) As IList(Of InfoMostrar)
        Dim valores As List(Of InfoMostrar) = New List(Of InfoMostrar)
        For Each item As KeyValuePair(Of String, String) In lista
            Dim infoMostrar As InfoMostrar = New InfoMostrar()
            infoMostrar.Valor = item.Key
            infoMostrar.Text = item.Value
            valores.Add(infoMostrar)
        Next
        Return valores
    End Function

    ''' <summary>
    ''' Llena el combo con los valores de un Enumerado.
    ''' </summary>
    ''' <param name="combo">Combo a llenar</param>
    ''' <param name="typeEnum">Tipo del Enumerado</param>
    ''' <remarks></remarks>
    Public Sub LlenarValoresDesde(ByRef combo As ComboBox, ByRef typeEnum As Type)
        combo.DataSource = GetValoresMostrarPara(typeEnum)
        combo.DisplayMember = "Text"
        combo.ValueMember = "Valor"
    End Sub

    ''' <summary>
    ''' Llena el combo con los valores procedentes de una Lista de objetos de tipo KeyValuePair(Of String, String).
    ''' </summary>
    ''' <param name="combo">Combo a llenar</param>
    ''' <param name="lista">Lista de objetos desde la cual se va a llenar el combo</param>
    ''' <remarks></remarks>
    Public Sub LlenarValoresDesdeLista(ByRef combo As ComboBox, ByRef lista As IList(Of KeyValuePair(Of String, String)))
        combo.DataSource = GetValoresMostrarDesdeLista(lista)
        combo.DisplayMember = "Text"
        combo.ValueMember = "Valor"
    End Sub

    ''' <summary>
    ''' Permite obtener todos los controles que se encuentran dentro de un control determinado, dada una lista de tipos de controles de interfaz de usuario.
    ''' </summary>
    ''' <param name="control">Control contenedor</param>
    ''' <param name="tipos">Tipos de controles a filtrar</param>
    ''' <returns>lista con los controles encontrados</returns>
    ''' <remarks></remarks>
    Public Function ObtenerControles(ByRef control As Control, ByRef tipos As List(Of String)) As List(Of Control)
        Dim resultado As List(Of Control) = New List(Of Control)
        Dim controles As IEnumerator = control.Controls.GetEnumerator()
        While controles.MoveNext()
            Dim cont As Control = controles.Current
            Dim tipo As String = TypeName(cont)
            If tipos.Contains(tipo) Then
                resultado.Add(cont)
            End If
        End While

        Return resultado
    End Function

    ''' <summary>
    ''' Permite realizar las validaciones sobre las diferentes secciones de un formulario.
    ''' </summary>
    ''' <param name="form">Formulario a validar</param>
    ''' <param name="secciones">Secciones del formulario a validar</param>
    ''' <param name="definicionesValidacion">Diccionario con las definiciones de validaciones para el formulario</param>
    ''' <returns>string con los mensajes de error producto de la validación</returns>
    ''' <remarks></remarks>
    Public Function ValidarFormulario(ByRef form As Form, ByRef secciones As List(Of String), ByRef definicionesValidacion As Dictionary(Of String, Validacion)) As String
        Dim mensajeFinal As StringBuilder = New StringBuilder()
        Dim listaTipos As List(Of String) = New List(Of String) From {GetType(ComboBox).Name, GetType(TextBox).Name, GetType(NumericUpDown).Name, GetType(DataGridView).Name}
        Dim controlesEntrada As List(Of Control) = ObtenerControles(form, listaTipos)
        For Each control As Control In controlesEntrada
            ' obtener el valor del control
            Dim valor As String = ""
            If TypeOf control Is ComboBox Then
                valor = CType(control, ComboBox).SelectedValue.ToString()
            ElseIf TypeOf control Is TextBox Then
                valor = CType(control, TextBox).Text
            ElseIf TypeOf control Is NumericUpDown Then
                valor = CType(control, NumericUpDown).Value.ToString()
            ElseIf TypeOf control Is DataGridView Then
                Dim rowCount As Integer = CType(control, DataGridView).RowCount
                If rowCount = 0 Then
                    valor = ""
                Else
                    valor = rowCount.ToString()
                End If
            End If

            Dim nombreControl As String = control.Name
            If definicionesValidacion.ContainsKey(nombreControl) AndAlso secciones.Contains(definicionesValidacion.Item(nombreControl).Seccion) Then
                Dim defVal As Validacion = definicionesValidacion.Item(nombreControl)
                Dim nombreReglas = defVal.Reglas.Keys
                For Each nombreRegla As String In nombreReglas
                    Dim mensaje As String = ""
                    If nombreRegla = "req" Then
                        If String.IsNullOrEmpty(valor) Then
                            mensaje = String.Format("El campo {0} es requrido.", defVal.LabelCampo)
                            mensajeFinal.AppendLine(mensaje)
                            Exit For
                        End If
                    ElseIf nombreRegla = "num" Then
                        Try
                            If Not String.IsNullOrEmpty(valor) Then
                                Dim valorDec As Decimal = Decimal.Parse(valor)
                            End If
                        Catch ex As Exception
                            mensaje = String.Format("El campo {0} solo admite valores numéricos.", defVal.LabelCampo)
                            mensajeFinal.AppendLine(mensaje)
                            Exit For
                        End Try
                    ElseIf nombreRegla = "min" Then
                        If Not String.IsNullOrEmpty(valor) Then
                            Dim valorDec As Decimal = Decimal.Parse(valor)
                            Dim decComp As Decimal = Decimal.Parse(defVal.Reglas.Item(nombreRegla).ToString())
                            If valorDec < decComp Then
                                mensaje = String.Format("El valor mínimo para el campo {0} es {1}.", defVal.LabelCampo, decComp)
                                mensajeFinal.AppendLine(mensaje)
                                Exit For
                            End If
                        End If
                    ElseIf nombreRegla = "rut" Then
                        If Not String.IsNullOrEmpty(valor) Then
                            If Not ValidadorRUT.EsValido(valor) Then
                                mensaje = String.Format("El valor del campo {0} no es un RUT válido.", defVal.LabelCampo)
                                mensajeFinal.AppendLine(mensaje)
                                Exit For
                            End If
                        End If
                    End If
                Next
            End If

        Next

        Return mensajeFinal.ToString()
    End Function



End Module
