﻿Imports DocumentoDTE.SiiDte

''' <summary>
''' Asistente para el trabajo con los tipos de datos enumerados.
''' </summary>
''' <remarks></remarks>
Public Module AsistenteEnum

    ''' <summary>
    ''' Obtiene una lista con los tipos de documentos.
    ''' </summary>
    ''' <returns>Lista de objetos KeyValuePair(Of String, String) (código, nombre) que representan los tipos de documentos.</returns>
    ''' <remarks></remarks>
    Public Function ListaTipoDocumento() As List(Of KeyValuePair(Of String, String))
        Dim resultado As List(Of KeyValuePair(Of String, String))
        resultado = New List(Of KeyValuePair(Of String, String))

        resultado.Add(New KeyValuePair(Of String, String)("33", "Factura Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("34", "Factura No Afecta o Exenta Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("39", "Boleta Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("41", "Boleta Exenta Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("43", "Liquidación-Factura Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("46", "Factura de Compra Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("52", "Guía de Despacho Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("56", "Nota de Débito Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("61", "Nota de Crédito Electrónica"))
        resultado.Add(New KeyValuePair(Of String, String)("110", "Factura de Exportación"))
        resultado.Add(New KeyValuePair(Of String, String)("111", "Nota de Débito de Exportación"))
        resultado.Add(New KeyValuePair(Of String, String)("112", "Nota de Crédito de Exportación"))
        Return resultado
    End Function

    ''' <summary>
    ''' Obtiene una lista con las unidades de medida.
    ''' </summary>
    ''' <returns>Lista de objetos KeyValuePair(Of String, String) (sigla, nombre) que representan las unidades de medida.</returns>
    ''' <remarks></remarks>
    Public Function ListaUnidadMedida() As List(Of KeyValuePair(Of String, String))
        Dim resultado As List(Of KeyValuePair(Of String, String))
        resultado = New List(Of KeyValuePair(Of String, String))

        resultado.Add(New KeyValuePair(Of String, String)("und", "Unidad"))
        resultado.Add(New KeyValuePair(Of String, String)("t", "Tonelada"))
        resultado.Add(New KeyValuePair(Of String, String)("kg", "Kilogramo"))
        resultado.Add(New KeyValuePair(Of String, String)("lb", "Libra"))
        resultado.Add(New KeyValuePair(Of String, String)("g", "Gramo"))
        resultado.Add(New KeyValuePair(Of String, String)("oz", "Onza"))
        resultado.Add(New KeyValuePair(Of String, String)("km", "Kilometro"))
        resultado.Add(New KeyValuePair(Of String, String)("m", "Metro"))
        resultado.Add(New KeyValuePair(Of String, String)("cm", "Centimetro"))
        Return resultado
    End Function

    ''' <summary>
    ''' Obtiene la descripción para mostrar de un valor de un tipo de dato enumerado.
    ''' </summary>
    ''' <param name="tipoenum">Tipo del enummerado</param>
    ''' <param name="valor">Valor al cual se le va a obtener la descripción.</param>
    ''' <returns>Descripción del valor</returns>
    ''' <remarks></remarks>
    Public Function ObtenerDescripcion(ByRef tipoenum As Type, ByVal valor As String) As String
        Dim resultado As String = "n"

        If GetType(DocumentoDTE.SiiDte.Enumerations.Referencia_CodRef) Is tipoenum Then
            Select Case valor
                Case "1"
                    resultado = "Anula Documento de Referencia"
                Case "2"
                    resultado = "Corrige Texto del Documento de Referencia"
                Case "3"
                    resultado = "Corrige Montos"
            End Select
        ElseIf GetType(DocumentoDTE.SiiDte.Enumerations.DetalleB_IndExe) Is tipoenum Then
            Select Case valor
                Case "1"
                    resultado = "El Producto o Servicio NO ESTA Afecto a IVA"
                Case "2"
                    resultado = "El Producto o Servicio NO ES Facturable"
                Case "3"
                    resultado = "Garantia por Deposito/Envase"
                Case "4"
                    resultado = "El producto No Constituye Venta"
                Case "5"
                    resultado = "Item a Rebajar"
                Case "6"
                    resultado = "No facturables negativos"
            End Select
        ElseIf GetType(DocumentoDTE.SiiDte.Enumerations.Detalle_IndExe) Is tipoenum Then
            Select Case valor
                Case "1"
                    resultado = "El Producto o Servicio NO ESTA Afecto a IVA"
                Case "2"
                    resultado = "El Producto o Servicio NO ES Facturable"
                Case "3"
                    resultado = "Garantia por Deposito/Envase"
                Case "4"
                    resultado = "El producto No Constituye Venta"
                Case "5"
                    resultado = "Item a Rebajar"
                Case "6"
                    resultado = "No facturables negativos"
            End Select
        ElseIf GetType(DocumentoDTE.SiiDte.Enumerations.DetalleA_IndExe) Is tipoenum Then
            Select Case valor
                Case "1"
                    resultado = "El Producto o Servicio NO ESTA Afecto a IVA"
                Case "2"
                    resultado = "El Producto o Servicio NO ES Facturable"
                Case "3"
                    resultado = "Garantia por Deposito/Envase"
                Case "4"
                    resultado = "El producto No Constituye Venta"
                Case "5"
                    resultado = "Item a Rebajar"
                Case "6"
                    resultado = "No facturables negativos"
            End Select
        ElseIf GetType(DocumentoBoleta.SiiDte.Enumerations.Detalle_IndExe) Is tipoenum Then
            Select Case valor
                Case "1"
                    resultado = "Producto o Servicio es Exento o No Afecto"
                Case "2"
                    resultado = "El Producto o Servicio NO ES Facturable"
            End Select
        End If

        Return resultado
    End Function

End Module
