﻿Imports DocumentoBoleta.SiiDte
Imports LiquidTechnologies.Runtime.Net20

''' <summary>
''' Representa la estructura de una Boleta.
''' </summary>
''' <remarks></remarks>
Public Class Boleta
    Implements IFactura

    ''' <summary>
    ''' Lista de Detalles de una Boleta.
    ''' </summary>
    ''' <remarks></remarks>
    Public ListaDetalles As List(Of Detalle)

    ''' <summary>
    ''' Lista de Referencias de una Boleta.
    ''' </summary>
    ''' <remarks></remarks>
    Public ListaReferencias As List(Of Referencia)

    ''' <summary>
    ''' DTE Boleta.
    ''' </summary>
    ''' <remarks></remarks>
    Private dte As DocumentoBoleta.SiiDte.BOLETADefType

    ''' <summary>
    ''' DTE Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetDTE() As DocumentoBoleta.SiiDte.BOLETADefType
        Get
            Return dte
        End Get
    End Property

    ''' <summary>
    ''' Estructura del Documento para la Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetValor() As DocumentoBoleta.SiiDte.Documento
        Get
            Return dte.Documento
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/IdDoc de Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetIdDoc() As IdDoc
        Get
            Return dte.Documento.Encabezado.IdDoc
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Emisor de Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetEmisor() As Emisor
        Get
            Return dte.Documento.Encabezado.Emisor
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Receptor de Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetReceptor() As Receptor
        Get
            Return dte.Documento.Encabezado.Receptor
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Totales de Boleta.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetTotales() As Totales
        Get
            Return dte.Documento.Encabezado.Totales
        End Get
    End Property

    ''' <summary>
    ''' Crea un objeto Boleta.
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        ListaDetalles = New List(Of Detalle)
        ListaReferencias = New List(Of Referencia)

        dte = New BOLETADefType()
        dte.Documento = New DocumentoBoleta.SiiDte.Documento()
        dte.Documento.Encabezado = New Encabezado()
        dte.Documento.Encabezado.IdDoc = New IdDoc()
        dte.Documento.Encabezado.Emisor = New Emisor()
        dte.Documento.Encabezado.Receptor = New Receptor()
        dte.Documento.Encabezado.Totales = New Totales()
    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/IdDoc.
    ''' </summary>
    ''' <param name="TipoDTE">Tipo de DTE</param>
    ''' <param name="Folio">Folio del documento</param>
    ''' <param name="FchEmis">Fecha de emisión</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoIdDoc(ByVal TipoDTE As String, ByRef Folio As Long, ByRef FchEmis As DateTime)
        Dim IdDoc As IdDoc = GetIdDoc()
        IdDoc.Folio = Folio
        IdDoc.TipoDTE = DocumentoBoleta.SiiDte.Enumerations.DTETypeFromString(TipoDTE)
        IdDoc.FchEmis = New LiquidTechnologies.Runtime.Net20.XmlDateTime(FchEmis)
    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Emisor.
    ''' </summary>
    ''' <param name="RUTEmisor">RUT del emisor</param>
    ''' <param name="RznSoc">Razón social del emisor</param>
    ''' <param name="GiroEmis">Giro del emisor</param>
    ''' <param name="DirOrigen">Dirección origen del emisor</param>
    ''' <param name="CmnaOrigen">Comuna origen del emisor</param>
    ''' <param name="CiudadOrigen">Ciudad origen del emisor</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoEmisor(ByVal RUTEmisor As String, ByVal RznSoc As String, ByVal GiroEmis As String,
                            ByVal DirOrigen As String, ByVal CmnaOrigen As String, ByVal CiudadOrigen As String)
        Dim Emisor As Emisor = GetEmisor()
        Emisor.RUTEmisor = RUTEmisor
        Emisor.RznSocEmisor = RznSoc
        Emisor.GiroEmisor = GiroEmis
        Emisor.DirOrigen = DirOrigen
        Emisor.CmnaOrigen = CmnaOrigen
        Emisor.CiudadOrigen = CiudadOrigen

    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Receptor.
    ''' </summary>
    ''' <param name="RUTRecep">RUT del Receptor</param>
    ''' <param name="RznSocRecep">Razón social del Receptor</param>
    ''' <param name="DirRecep">Dirección del Receptor</param>
    ''' <param name="CmnaRecep">Comuna del Receptor</param>
    ''' <param name="CiudadRecep">Ciudad del Recepto</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoReceptor(ByVal RUTRecep As String, ByVal RznSocRecep As String,
                              ByVal DirRecep As String, ByVal CmnaRecep As String, ByVal CiudadRecep As String)
        Dim Receptor As Receptor = GetReceptor()
        Receptor.RUTRecep = RUTRecep
        Receptor.RznSocRecep = RznSocRecep
        Receptor.DirRecep = DirRecep
        Receptor.CmnaRecep = CmnaRecep
        Receptor.CiudadRecep = CiudadRecep
    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Totales.
    ''' </summary>
    ''' <param name="MntTotal">Monto total</param>
    ''' <param name="MntExe">Monto exento</param>
    ''' <param name="MntNeto">Monto neto</param>
    ''' <param name="IVA">IVA</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoTotales(ByVal MntTotal As Long, ByVal MntExe As Long, ByVal MntNeto As Long, ByVal IVA As Long)
        Dim Totales As Totales = GetTotales()
        Totales.MntTotal = MntTotal
        Totales.MntExe = MntExe
        Totales.MntNeto = MntNeto
        If IVA > 0 Then
            Totales.IVA = IVA
        End If
    End Sub

    ''' <summary>
    ''' Adiciona un item a los detalles del documento.
    ''' </summary>
    ''' <param name="NmbItem">Nombre del item</param>
    ''' <param name="MontoItem">Monto del item</param>
    ''' <param name="IndExe">Indicador exento</param>
    ''' <param name="DscItem">Descripción del item</param>
    ''' <param name="QtyItem">Cantidad del item</param>
    ''' <param name="UnmdItem">Unidad de medida del item</param>
    ''' <param name="PrcItem">Precio del item</param>
    ''' <param name="ListaCdgItem">Lista de códigos del item</param>
    ''' <remarks></remarks>
    Public Sub AdicionarDetalle(ByVal NmbItem As String, ByVal MontoItem As Long, ByVal IndExe As String, ByVal DscItem As String,
                         ByVal QtyItem As Decimal, ByVal UnmdItem As String, ByVal PrcItem As Decimal, ByRef ListaCdgItem As IList(Of CdgItem))
        ActualizarDetalleEnIndice(NmbItem, MontoItem, IndExe, DscItem, QtyItem, UnmdItem, PrcItem, ListaCdgItem, -1)
    End Sub

    ''' <summary>
    ''' Actualiza un item de los detalles del documento dada la posición del mismo.
    ''' </summary>
    ''' <param name="NmbItem">Nombre del item</param>
    ''' <param name="MontoItem">Monto del item</param>
    ''' <param name="IndExe">Indicador exento</param>
    ''' <param name="DscItem">Descripción del item</param>
    ''' <param name="QtyItem">Cantidad del item</param>
    ''' <param name="UnmdItem">Unidad de medida del item</param>
    ''' <param name="PrcItem">Precio del item</param>
    ''' <param name="ListaCdgItem">Lista de códigos del item</param>
    ''' <param name="indice">indice del item a actualizar</param>
    ''' <remarks></remarks>
    Public Sub ActualizarDetalleEnIndice(ByVal NmbItem As String, ByVal MontoItem As Long, ByVal IndExe As String, ByVal DscItem As String,
                         ByVal QtyItem As Decimal, ByVal UnmdItem As String, ByVal PrcItem As Decimal, ByRef ListaCdgItem As IList(Of CdgItem), ByVal indice As Integer)
        Dim detL As DocumentoBoleta.SiiDte.Detalle = New DocumentoBoleta.SiiDte.Detalle()
        detL.NroLinDet = New BigInteger(ListaDetalles.Count + 1)
        detL.NmbItem = NmbItem
        detL.MontoItem = MontoItem
        If IndExe <> "" Then
            detL.IndExe = Enumerations.Detalle_IndExeFromString(IndExe)
        End If
        detL.DscItem = DscItem
        detL.QtyItem = QtyItem
        detL.UnmdItem = UnmdItem
        detL.PrcItem = PrcItem

        detL.CdgItem.Clear()
        For Each cdgItemL As CdgItem In ListaCdgItem
            detL.CdgItem.Add(cdgItemL)
        Next

        If indice > -1 Then
            ListaDetalles.Item(indice) = detL
        Else
            ListaDetalles.Add(detL)
        End If

    End Sub

    ''' <summary>
    ''' Elimina un item de la lista de detalles.
    ''' </summary>
    ''' <param name="indice">indice del item a eliminar</param>
    ''' <remarks></remarks>
    Public Sub EliminarDetalleEnIndice(ByVal indice As Integer)
        ListaDetalles.RemoveAt(indice)
    End Sub

    ''' <summary>
    ''' Crea un objeto CdgItem.
    ''' </summary>
    ''' <param name="TpoCodigo">Tipo de código</param>
    ''' <param name="VlrCodigo">Valor del código</param>
    ''' <returns>Objeto de tipo CdgItem</returns>
    ''' <remarks></remarks>
    Public Function CrearDetalleCdgItem(ByVal TpoCodigo As String, ByVal VlrCodigo As String) As CdgItem
        Dim cdgItemL As CdgItem = New DocumentoBoleta.SiiDte.CdgItem()
        cdgItemL.TpoCodigo = TpoCodigo
        cdgItemL.VlrCodigo = VlrCodigo
        Return cdgItemL
    End Function

    ''' <summary>
    ''' Adiciona una Referencia a la lista de Referencias.
    ''' </summary>
    ''' <param name="CodRef">Código de la Referencia</param>
    ''' <param name="RazonRef">Razón de la Referencia</param>
    ''' <remarks></remarks>
    Public Sub AdicionarReferencia(ByVal CodRef As String, ByVal RazonRef As String)
        ActualizarReferenciaEnIndice(CodRef, RazonRef, -1)
    End Sub

    ''' <summary>
    ''' Actualiza una Referencia a la lista de Referencias dado el indice de la misma.
    ''' </summary>
    ''' <param name="CodRef">Código de la Referencia</param>
    ''' <param name="RazonRef">Razón de la Referencia</param>
    ''' <param name="indice">indice de la Referencia en la lista</param>
    ''' <remarks></remarks>
    Public Sub ActualizarReferenciaEnIndice(ByVal CodRef As String, ByVal RazonRef As String, ByVal indice As Integer)
        Dim referenciaL As Referencia = New Referencia()
        referenciaL.NroLinRef = New BigInteger(ListaReferencias.Count + 1)
        referenciaL.CodRef = CodRef
        referenciaL.RazonRef = RazonRef

        If indice > -1 Then
            ListaReferencias.Item(indice) = referenciaL
        Else
            ListaReferencias.Add(referenciaL)
        End If
    End Sub

    ''' <summary>
    ''' Elimina una Referencia de la lista de Referencias dado el indice de la misma.
    ''' </summary>
    ''' <param name="indice">indice de la Referencia en la lista</param>
    ''' <remarks></remarks>
    Public Sub EliminarReferenciaEnIndice(ByVal indice As Integer)
        ListaReferencias.RemoveAt(indice)
    End Sub

    ''' <summary>
    ''' Obtiene el XML representativo de la Boleta.
    ''' </summary>
    ''' <returns>string con el XML de la Boleta</returns>
    ''' <remarks></remarks>
    Public Function ObtenerXml() As String Implements IFactura.ObtenerXml
        Dim documento As DocumentoBoleta.SiiDte.Documento = GetValor()
        Dim indice As Integer = 0

        documento.Detalle.Clear()
        For Each detL As Detalle In ListaDetalles
            indice += 1
            detL.NroLinDet = New BigInteger(indice)
            documento.Detalle.Add(detL)
        Next
        indice = 0
        documento.Referencia.Clear()
        For Each referenciaL As Referencia In ListaReferencias
            indice += 1
            referenciaL.NroLinRef = New BigInteger(indice)
            documento.Referencia.Add(referenciaL)
        Next

        Return GeneradorXmlBoleta.ObtenerXML(dte)
    End Function

End Class
