﻿Imports DocumentoDTE.SiiDte
Imports LiquidTechnologies.Runtime.Net20

''' <summary>
''' Representa la estructura de un Documento.
''' </summary>
''' <remarks></remarks>
Public Class Documento
    Implements IFactura

    ''' <summary>
    ''' Lista de Detalles de un Documento.
    ''' </summary>
    ''' <remarks></remarks>
    Public ListaDetalles As List(Of Detalle)

    ''' <summary>
    ''' Lista de Referencias de un Documento.
    ''' </summary>
    ''' <remarks></remarks>
    Public ListaReferencias As List(Of Referencia)

    ''' <summary>
    ''' DTE Documento.
    ''' </summary>
    ''' <remarks></remarks>
    Private dteVal As DocumentoDTE.SiiDte.DTEDefType

    ''' <summary>
    ''' DTE Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property DTE() As DocumentoDTE.SiiDte.DTEDefType
        Get
            Return dteVal
        End Get
    End Property

    ''' <summary>
    ''' Estructura del Documento para el Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Valor() As DocumentoDTE.SiiDte.Documento
        Get
            Return DTE.DTE_Choice.Documento
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/IdDoc de Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IdDoc() As IdDoc
        Get
            Return DTE.DTE_Choice.Documento.Encabezado.IdDoc
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Emisor de Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Emisor() As Emisor
        Get
            Return DTE.DTE_Choice.Documento.Encabezado.Emisor
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Receptor de Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Receptor() As Receptor
        Get
            Return DTE.DTE_Choice.Documento.Encabezado.Receptor
        End Get
    End Property

    ''' <summary>
    ''' Encabezado/Totales de Documento.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Totales() As Totales
        Get
            Return DTE.DTE_Choice.Documento.Encabezado.Totales
        End Get
    End Property

    ''' <summary>
    ''' Crea un Objeto Documento.
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        ListaDetalles = New List(Of Detalle)
        ListaReferencias = New List(Of Referencia)

        dteVal = New DTEDefType()
        dteVal.DTE_Choice = New DTE_Choice()
        dteVal.DTE_Choice.Documento = New DocumentoDTE.SiiDte.Documento()
        dteVal.DTE_Choice.Documento.Encabezado = New Encabezado()
        dteVal.DTE_Choice.Documento.Encabezado.IdDoc = New IdDoc()
        dteVal.DTE_Choice.Documento.Encabezado.Emisor = New Emisor()
        dteVal.DTE_Choice.Documento.Encabezado.Receptor = New Receptor()
        dteVal.DTE_Choice.Documento.Encabezado.Totales = New Totales()

    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/IdDoc.
    ''' </summary>
    ''' <param name="TipoDTE">Tipo de DTE</param>
    ''' <param name="Folio">Folio del documento</param>
    ''' <param name="FchEmis">Fecha de emisión</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoIdDoc(ByVal TipoDTE As String, ByRef Folio As Long, ByRef FchEmis As DateTime)
        Dim IdDoc As IdDoc = Me.IdDoc()
        IdDoc.Folio = Folio
        IdDoc.TipoDTE = DocumentoDTE.SiiDte.Enumerations.DTETypeFromString(TipoDTE)
        IdDoc.FchEmis = New LiquidTechnologies.Runtime.Net20.XmlDateTime(FchEmis)
    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Emisor.
    ''' </summary>
    ''' <param name="RUTEmisor">RUT del emiso</param>
    ''' <param name="RznSoc">Razón social del emisor</param>
    ''' <param name="GiroEmis">Giro del emisor</param>
    ''' <param name="Actectos">Actectos del emisor</param>
    ''' <param name="DirOrigen">Dirección origen del emisor</param>
    ''' <param name="CmnaOrigen">Comuna origen del emisor</param>
    ''' <param name="CiudadOrigen">Ciudad</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoEmisor(ByVal RUTEmisor As String, ByVal RznSoc As String, ByVal GiroEmis As String,
                            ByVal Actectos As String, ByVal DirOrigen As String, ByVal CmnaOrigen As String, ByVal CiudadOrigen As String)
        Dim Emisor As Emisor = Me.Emisor()
        Emisor.RUTEmisor = RUTEmisor
        Emisor.RznSoc = RznSoc
        Emisor.GiroEmis = GiroEmis
        Emisor.DirOrigen = DirOrigen

        Emisor.Acteco.Clear()
        Dim item As String
        For Each item In Actectos.Split(",")
            Dim acteco As Long = 0
            Long.TryParse(item, acteco)
            Emisor.Acteco.Add(New BigInteger(acteco))
        Next
        Emisor.CmnaOrigen = CmnaOrigen
        Emisor.CiudadOrigen = CiudadOrigen

    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Receptor.
    ''' </summary>
    ''' <param name="RUTRecep">RUT del Receptor</param>
    ''' <param name="RznSocRecep">Razón social del Receptor</param>
    ''' <param name="GiroRecep">Giro del Receptor</param>
    ''' <param name="DirRecep">Dirección del Receptor</param>
    ''' <param name="CmnaRecep">Comuna del Receptor</param>
    ''' <param name="CiudadRecep">Ciudad del Receptor</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoReceptor(ByVal RUTRecep As String, ByVal RznSocRecep As String, ByVal GiroRecep As String,
                              ByVal DirRecep As String, ByVal CmnaRecep As String, ByVal CiudadRecep As String)
        Dim Receptor As Receptor = Me.Receptor()
        Receptor.RUTRecep = RUTRecep
        Receptor.RznSocRecep = RznSocRecep
        Receptor.GiroRecep = GiroRecep
        Receptor.DirRecep = DirRecep
        Receptor.CmnaRecep = CmnaRecep
        Receptor.CiudadRecep = CiudadRecep
    End Sub

    ''' <summary>
    ''' Establece los valores para Encabezado/Totales.
    ''' </summary>
    ''' <param name="MntTotal">Monto total</param>
    ''' <param name="MntExe">Monto exento</param>
    ''' <param name="MntNeto">Monto neto</param>
    ''' <param name="IVA">IVA</param>
    ''' <remarks></remarks>
    Public Sub SetEncabezadoTotales(ByVal MntTotal As Long, ByVal MntExe As Long, ByVal MntNeto As Long, ByVal IVA As Long)
        Dim Totales As Totales = Me.Totales()
        Totales.MntTotal = MntTotal
        Totales.MntExe = MntExe
        Totales.MntNeto = MntNeto
        If IVA > 0 Then
            Totales.IVA = IVA
        End If
    End Sub

    ''' <summary>
    ''' Adiciona un item a los detalles del documento.
    ''' </summary>
    ''' <param name="NmbItem">Nombre del item</param>
    ''' <param name="MontoItem">Monto del item</param>
    ''' <param name="IndExe">Indicador exento</param>
    ''' <param name="DscItem">Descripción del item</param>
    ''' <param name="QtyItem">Cantidad del item</param>
    ''' <param name="UnmdItem">Unidad de medida del item</param>
    ''' <param name="PrcItem">Precio del item</param>
    ''' <param name="ListaCdgItem">Lista de códigos del item</param>
    ''' <remarks></remarks>
    Public Sub AdicionarDetalle(ByVal NmbItem As String, ByVal MontoItem As Long, ByVal IndExe As String, ByVal DscItem As String,
                         ByVal QtyItem As Decimal, ByVal UnmdItem As String, ByVal PrcItem As Decimal, ByRef ListaCdgItem As IList(Of CdgItem))
        ActualizarDetalleEnIndice(NmbItem, MontoItem, IndExe, DscItem, QtyItem, UnmdItem, PrcItem, ListaCdgItem, -1)
    End Sub

    ''' <summary>
    ''' Actualiza un item de los detalles del documento dada la posición del mismo.
    ''' </summary>
    ''' <param name="NmbItem">Nombre del item</param>
    ''' <param name="MontoItem">Monto del item</param>
    ''' <param name="IndExe">Indicador exento</param>
    ''' <param name="DscItem">Descripción del item</param>
    ''' <param name="QtyItem">Cantidad del item</param>
    ''' <param name="UnmdItem">Unidad de medida del item</param>
    ''' <param name="PrcItem">Precio del item</param>
    ''' <param name="ListaCdgItem">Lista de códigos del item</param>
    ''' <param name="indice">indice del item a actualizar</param>
    ''' <remarks></remarks>
    Public Sub ActualizarDetalleEnIndice(ByVal NmbItem As String, ByVal MontoItem As Long, ByVal IndExe As String, ByVal DscItem As String,
                         ByVal QtyItem As Decimal, ByVal UnmdItem As String, ByVal PrcItem As Decimal, ByRef ListaCdgItem As IList(Of CdgItem), ByVal indice As Integer)
        Dim detL As DocumentoDTE.SiiDte.Detalle = New DocumentoDTE.SiiDte.Detalle()
        detL.NroLinDet = New BigInteger(ListaDetalles.Count + 1)
        detL.NmbItem = NmbItem
        detL.MontoItem = MontoItem
        If IndExe <> "" Then
            detL.IndExe = Enumerations.Detalle_IndExeFromString(IndExe)
        End If
        detL.DscItem = DscItem
        detL.QtyItem = QtyItem
        detL.UnmdItem = UnmdItem
        detL.PrcItem = PrcItem

        detL.CdgItem.Clear()
        For Each cdgItemL As CdgItem In ListaCdgItem
            detL.CdgItem.Add(cdgItemL)
        Next

        If indice > -1 Then
            ListaDetalles.Item(indice) = detL
        Else
            ListaDetalles.Add(detL)
        End If

    End Sub

    ''' <summary>
    ''' Elimina un item de la lista de detalles.
    ''' </summary>
    ''' <param name="indice">indice del item a eliminar</param>
    ''' <remarks></remarks>
    Public Sub EliminarDetalleEnIndice(ByVal indice As Integer)
        ListaDetalles.RemoveAt(indice)
    End Sub

    ''' <summary>
    ''' Crea un objeto CdgItem.
    ''' </summary>
    ''' <param name="TpoCodigo">Tipo de código</param>
    ''' <param name="VlrCodigo">Valor del código</param>
    ''' <returns>Objeto de tipo CdgItem</returns>
    ''' <remarks></remarks>
    Public Function CrearDetalleCdgItem(ByVal TpoCodigo As String, ByVal VlrCodigo As String) As CdgItem
        Dim cdgItemL As CdgItem = New DocumentoDTE.SiiDte.CdgItem()
        cdgItemL.TpoCodigo = TpoCodigo
        cdgItemL.VlrCodigo = VlrCodigo
        Return cdgItemL
    End Function

    ''' <summary>
    ''' Adiciona una Referencia a la lista de Referencias.
    ''' </summary>
    ''' <param name="TpoDocRef">Tipo de documento de la Referencia</param>
    ''' <param name="FolioRef">Folio de la Referencia</param>
    ''' <param name="FchRef">Fecha de la Referencia</param>
    ''' <param name="RazonRef">Razón de la Referencia</param>
    ''' <remarks></remarks>
    Public Sub AdicionarReferencia(ByVal TpoDocRef As String, ByVal FolioRef As String, ByVal FchRef As DateTime, ByVal RazonRef As String)
        ActualizarReferenciaEnIndice(TpoDocRef, FolioRef, FchRef, RazonRef, -1)
    End Sub

    ''' <summary>
    ''' Actualiza una Referencia a la lista de Referencias dado el indice de la misma.
    ''' </summary>
    ''' <param name="TpoDocRef">Tipo de documento de la Referencia</param>
    ''' <param name="FolioRef">Folio de la Referencia</param>
    ''' <param name="FchRef">Fecha de la Referencia</param>
    ''' <param name="RazonRef">Razón de la Referencia</param>
    ''' <param name="indice">índice de la referencia</param>
    ''' <remarks></remarks>
    Public Sub ActualizarReferenciaEnIndice(ByVal TpoDocRef As String, ByVal FolioRef As String, ByVal FchRef As DateTime, ByVal RazonRef As String, ByVal indice As Integer)
        Dim referenciaL As Referencia = New Referencia()
        referenciaL.NroLinRef = New BigInteger(ListaReferencias.Count + 1)
        referenciaL.TpoDocRef = TpoDocRef
        referenciaL.FolioRef = FolioRef
        referenciaL.FchRef = New LiquidTechnologies.Runtime.Net20.XmlDateTime(FchRef)
        referenciaL.RazonRef = RazonRef

        If indice > -1 Then
            ListaReferencias.Item(indice) = referenciaL
        Else
            ListaReferencias.Add(referenciaL)
        End If
    End Sub

    ''' <summary>
    ''' Elimina una Referencia de la lista de Referencias dado el indice de la misma.
    ''' </summary>
    ''' <param name="indice">indice de la Referencia en la lista</param>
    ''' <remarks></remarks>
    Public Sub EliminarReferenciaEnIndice(ByVal indice As Integer)
        ListaReferencias.RemoveAt(indice)
    End Sub

    ''' <summary>
    ''' Obtiene el XML representativo del Documento.
    ''' </summary>
    ''' <returns>string con el XML del Documento</returns>
    ''' <remarks></remarks>
    Public Function ObtenerXml() As String Implements IFactura.ObtenerXml
        Dim documento As DocumentoDTE.SiiDte.Documento = Valor()
        Dim indice As Integer = 0

        documento.Detalle.Clear()
        For Each detL As Detalle In ListaDetalles
            indice += 1
            detL.NroLinDet = New BigInteger(indice)
            documento.Detalle.Add(detL)
        Next
        indice = 0
        documento.Referencia.Clear()
        For Each referenciaL As Referencia In ListaReferencias
            indice += 1
            referenciaL.NroLinRef = New BigInteger(indice)
            documento.Referencia.Add(referenciaL)
        Next

        Return GeneradorXmlDTE.ObtenerXML(DTE)
    End Function


End Class
