﻿Imports LiquidTechnologies.Runtime.Net20
Imports System.IO
Imports System.Xml
Imports System.Text

''' <summary>
''' Clase utilitaria para generar el XML de los documentos de tipo Boleta.
''' </summary>
''' <remarks></remarks>
Module GeneradorXmlBoleta

    ''' <summary>
    ''' Obtiene el XML de una boleta.
    ''' </summary>
    ''' <param name="boleta">Boleta a la cual se le va a obtener el XML</param>
    ''' <returns>string que representa el XML de la Boleta</returns>
    ''' <remarks></remarks>
    Public Function ObtenerXML(ByRef boleta As DocumentoBoleta.SiiDte.BOLETADefType) As String
        boleta.Version = 1
        LlenarDocumento(boleta)

        Dim xsc As XmlSerializationContext = New LiquidTechnologies.Runtime.Net20.XmlSerializationContext()
        xsc.NamespaceAliases.Add("ABCXYZ", boleta.Namespace)
        Dim stream As MemoryStream = New MemoryStream()
        boleta.ToXmlStream(stream, True, Formatting.Indented, Encoding.GetEncoding("ISO-8859-1"), LiquidTechnologies.Runtime.Net20.EOLType.CRLF, xsc)
        Dim xmlBytes() As Byte = stream.ToArray()
        Dim xml As String = Encoding.UTF8.GetString(xmlBytes)
        Dim xmlStr As String = xml.Replace(":ABCXYZ", "").Replace("ABCXYZ:", "").Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for GDE S.A. (1 * Developer Bundle)-->\r\n", "").Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for SOUTH CONSULTING SIGNATURE S.A. (1 * Developer Edition)-->\r\n", "").Replace("xmlns=\'http://www.sii.cl/SiiDte\' ", "").Replace("BOLETADefType", "DTE")

        Return xmlStr
    End Function

    ''' <summary>
    ''' Llena la estructura TED de un documento Boleta.
    ''' </summary>
    ''' <param name="boleta">Boleta a la cual se le va a llenar la estructura TED</param>
    ''' <remarks></remarks>
    Private Sub LlenarDocumento(ByRef boleta As DocumentoBoleta.SiiDte.BOLETADefType)
        Dim doc As DocumentoBoleta.SiiDte.Documento = boleta.Documento
        doc.ID = "ID_1"
        ' TED
        doc.TED = New DocumentoBoleta.SiiDte.TED()

        doc.TED.Version = "1.0"

        ' TED/DD
        doc.TED.DD = New DocumentoBoleta.SiiDte.DD()

        doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor
        doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE
        doc.TED.DD.F = doc.Encabezado.IdDoc.Folio
        doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis
        doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep
        doc.TED.DD.RSR = "RSR"
        doc.TED.DD.MNT = CType(doc.Encabezado.Totales.MntTotal.LongValue(), ULong)
        doc.TED.DD.IT1 = "IT1"
        ' TED/Fake CAF
        doc.TED.DD.CAF = New DocumentoBoleta.SiiDte.CAF()
        doc.TED.DD.CAF.DA = New DocumentoBoleta.SiiDte.DA()

        doc.TED.DD.CAF.DA.RE = "00000000-K"
        doc.TED.DD.CAF.DA.RS = "A"
        doc.TED.DD.CAF.DA.TD = DocumentoBoleta.SiiDte.Enumerations.DTEType.n39
        doc.TED.DD.CAF.DA.FA = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1)
        doc.TED.DD.CAF.DA.IDK = 0

        doc.TED.DD.CAF.DA.DA_Choice = New DocumentoBoleta.SiiDte.DA_Choice()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK = New DocumentoBoleta.SiiDte.RSAPK()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK.M = New LiquidTechnologies.Runtime.Net20.BinaryData()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK.E = New LiquidTechnologies.Runtime.Net20.BinaryData()

        doc.TED.DD.CAF.DA.RNG = New DocumentoBoleta.SiiDte.RNG()
        doc.TED.DD.CAF.DA.RNG.D = 1
        doc.TED.DD.CAF.DA.RNG.H = 1

        doc.TED.DD.CAF.FRMA = New DocumentoBoleta.SiiDte.FRMA()
        doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA"

        doc.TED.DD.TSTED = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

        doc.TED.FRMT = New DocumentoBoleta.SiiDte.FRMT()
        doc.TED.FRMT.Algoritmo = DocumentoBoleta.EnvioBOLETA_v11Lib.Enumerations.FRMT_Algoritmo.SHA1withRSA

        doc.TmstFirma = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

    End Sub

End Module
