﻿Imports DocumentoDTE.SiiDte
Imports LiquidTechnologies.Runtime.Net20
Imports System.IO
Imports System.Xml
Imports System.Text

''' <summary>
''' Clase utilitaria para generar el XML de los documentos DTE.
''' </summary>
''' <remarks></remarks>
Public Module GeneradorXmlDTE

    ''' <summary>
    ''' Obtiene el XML de un DTE.
    ''' </summary>
    ''' <param name="dte">DTE al cual se le va a obtener el XML</param>
    ''' <returns>string que representa el XML del DTE</returns>
    ''' <remarks></remarks>
    Public Function ObtenerXML(ByRef dte As DTEDefType) As String
        dte.Version = 1
        If Not dte.DTE_Choice.Documento Is Nothing Then
            LlenarDocumento(CType(dte.DTE_Choice.Documento, DocumentoDTE.SiiDte.Documento))
        ElseIf Not dte.DTE_Choice.Exportaciones Is Nothing Then
            LlenarExportaciones(CType(dte.DTE_Choice.Exportaciones, DocumentoDTE.SiiDte.Exportaciones))
        ElseIf Not dte.DTE_Choice.Liquidacion Is Nothing Then
            LlenarLiquidacion(CType(dte.DTE_Choice.Liquidacion, DocumentoDTE.SiiDte.Liquidacion))
        End If

        Dim xsc As XmlSerializationContext = New LiquidTechnologies.Runtime.Net20.XmlSerializationContext()
        xsc.NamespaceAliases.Add("ABCXYZ", dte.Namespace)
        Dim stream As MemoryStream = New MemoryStream()
        dte.ToXmlStream(stream, True, Formatting.Indented, Encoding.GetEncoding("ISO-8859-1"), LiquidTechnologies.Runtime.Net20.EOLType.CRLF, xsc)

        Dim xmlBytes() As Byte = stream.ToArray()
        Dim xml As String = Encoding.UTF8.GetString(xmlBytes)
        Dim xmlStr As String = xml.Replace(":ABCXYZ", "").Replace("ABCXYZ:", "").Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for GDE S.A. (1 * Developer Bundle)-->\r\n", "").Replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for SOUTH CONSULTING SIGNATURE S.A. (1 * Developer Edition)-->\r\n", "").Replace("xmlns=\'http://www.sii.cl/SiiDte\' ", "").Replace("DTEDefType", "DTE")

        Return xmlStr
    End Function

    ''' <summary>
    ''' Llena la estructura TED de un DTE de tipo Documento.
    ''' </summary>
    ''' <param name="doc">Documento al cual se le va a llenar la estructura TED</param>
    ''' <remarks></remarks>
    Private Sub LlenarDocumento(ByRef doc As DocumentoDTE.SiiDte.Documento)
        doc.ID = "ID_1"
        ' TED
        doc.TED = New DocumentoDTE.SiiDte.TED()
        doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor
        doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE
        doc.TED.DD.F = doc.Encabezado.IdDoc.Folio

        doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis
        doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep
        doc.TED.DD.RSR = "RSR"
        doc.TED.DD.MNT = CType(doc.Encabezado.Totales.MntTotal.LongValue(), ULong)
        doc.TED.DD.IT1 = "IT1"

        ' TED/Fake CAF
        doc.TED.DD.CAF = New DocumentoDTE.SiiDte.CAF()
        doc.TED.DD.CAF.DA = New DocumentoDTE.SiiDte.DA()

        doc.TED.DD.CAF.DA.RE = "00000000-K"
        doc.TED.DD.CAF.DA.RS = "A"
        doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.DTEType.n33
        doc.TED.DD.CAF.DA.FA = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1)
        doc.TED.DD.CAF.DA.IDK = 0

        doc.TED.DD.CAF.DA.DA_Choice = New DocumentoDTE.SiiDte.DA_Choice()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK = New DocumentoDTE.SiiDte.RSAPK()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK.M = New LiquidTechnologies.Runtime.Net20.BinaryData()
        doc.TED.DD.CAF.DA.DA_Choice.RSAPK.E = New LiquidTechnologies.Runtime.Net20.BinaryData()

        doc.TED.DD.CAF.DA.RNG = New DocumentoDTE.SiiDte.RNG()
        doc.TED.DD.CAF.DA.RNG.D = 1
        doc.TED.DD.CAF.DA.RNG.H = 1

        doc.TED.DD.CAF.FRMA = New DocumentoDTE.SiiDte.FRMA()
        doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA"

        doc.TED.DD.TSTED = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

        doc.TED.FRMT = New DocumentoDTE.SiiDte.FRMT()
        doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMT_Algoritmo.SHA1withRSA

        doc.TmstFirma = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

    End Sub

    ''' <summary>
    ''' Llena la estructura TED de un DTE de tipo Exportaciones.
    ''' </summary>
    ''' <param name="doc">Exportación a la cual se le va a llenar la estructura TED</param>
    ''' <remarks></remarks>
    Private Sub LlenarExportaciones(ByRef doc As DocumentoDTE.SiiDte.Exportaciones)
        doc.ID = "ID_1"

        ' TED
        doc.TED = New DocumentoDTE.SiiDte.TEDB()

        doc.TED.Version = "1.0"

        ' TED/DD
        doc.TED.DD = New DocumentoDTE.SiiDte.DDB()

        doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor
        doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE
        doc.TED.DD.F = doc.Encabezado.IdDoc.Folio
        doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis
        doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep
        doc.TED.DD.RSR = "RSR"
        doc.TED.DD.MNT = doc.Encabezado.Totales.MntTotal
        doc.TED.DD.IT1 = "IT1"

        ' TED/Fake CAF
        doc.TED.DD.CAF = New DocumentoDTE.SiiDte.CAFB()
        doc.TED.DD.CAF.DA = New DocumentoDTE.SiiDte.DAB()

        doc.TED.DD.CAF.DA.RE = "00000000-K"
        doc.TED.DD.CAF.DA.RS = "A"
        doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.EXPType.n110
        doc.TED.DD.CAF.DA.FA = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1)
        doc.TED.DD.CAF.DA.IDK = 0

        doc.TED.DD.CAF.DA.DA1_Choice = New DocumentoDTE.SiiDte.DA1_Choice()
        doc.TED.DD.CAF.DA.DA1_Choice.RSAPK = New DocumentoDTE.SiiDte.RSAPKB()
        doc.TED.DD.CAF.DA.DA1_Choice.RSAPK.M = New LiquidTechnologies.Runtime.Net20.BinaryData()
        doc.TED.DD.CAF.DA.DA1_Choice.RSAPK.E = New LiquidTechnologies.Runtime.Net20.BinaryData()

        doc.TED.DD.CAF.DA.RNG = New DocumentoDTE.SiiDte.RNGB()
        doc.TED.DD.CAF.DA.RNG.D = 1
        doc.TED.DD.CAF.DA.RNG.H = 1

        doc.TED.DD.CAF.FRMA = New DocumentoDTE.SiiDte.FRMAB()
        doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA"

        doc.TED.DD.TSTED = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

        doc.TED.FRMT = New DocumentoDTE.SiiDte.FRMTB()
        doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMTB_Algoritmo.SHA1withRSA

        doc.TmstFirma = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)
    End Sub

    ''' <summary>
    ''' Llena la estructura TED de un DTE de tipo Liquidación.
    ''' </summary>
    ''' <param name="doc">Liquidación a la cual se le va a llenar la estructura TED</param>
    ''' <remarks></remarks>
    Private Sub LlenarLiquidacion(ByRef doc As DocumentoDTE.SiiDte.Liquidacion)
        doc.ID = "ID_1"

        ' TED
        doc.TED = New DocumentoDTE.SiiDte.TEDA()

        doc.TED.Version = "1.0"

        ' TED/DD
        doc.TED.DD = New DocumentoDTE.SiiDte.DDA()

        doc.TED.DD.RE = doc.Encabezado.Emisor.RUTEmisor
        doc.TED.DD.TD = doc.Encabezado.IdDoc.TipoDTE
        doc.TED.DD.F = doc.Encabezado.IdDoc.Folio
        doc.TED.DD.FE = doc.Encabezado.IdDoc.FchEmis
        doc.TED.DD.RR = doc.Encabezado.Receptor.RUTRecep
        doc.TED.DD.RSR = "RSR"
        doc.TED.DD.MNT = CType(doc.Encabezado.Totales.MntTotal.LongValue(), ULong)
        doc.TED.DD.IT1 = "IT1"

        ' TED/Fake CAF
        doc.TED.DD.CAF = New DocumentoDTE.SiiDte.CAFA()
        doc.TED.DD.CAF.DA = New DocumentoDTE.SiiDte.DAA()

        doc.TED.DD.CAF.DA.RE = "00000000-K"
        doc.TED.DD.CAF.DA.RS = "A"
        doc.TED.DD.CAF.DA.TD = DocumentoDTE.SiiDte.Enumerations.LIQType.n43
        doc.TED.DD.CAF.DA.FA = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1)
        doc.TED.DD.CAF.DA.IDK = 0

        doc.TED.DD.CAF.DA.DA0_Choice = New DocumentoDTE.SiiDte.DA0_Choice()
        doc.TED.DD.CAF.DA.DA0_Choice.RSAPK = New DocumentoDTE.SiiDte.RSAPKA()
        doc.TED.DD.CAF.DA.DA0_Choice.RSAPK.M = New LiquidTechnologies.Runtime.Net20.BinaryData()
        doc.TED.DD.CAF.DA.DA0_Choice.RSAPK.E = New LiquidTechnologies.Runtime.Net20.BinaryData()

        doc.TED.DD.CAF.DA.RNG = New DocumentoDTE.SiiDte.RNGA()
        doc.TED.DD.CAF.DA.RNG.D = 1
        doc.TED.DD.CAF.DA.RNG.H = 1

        doc.TED.DD.CAF.FRMA = New DocumentoDTE.SiiDte.FRMAA()
        doc.TED.DD.CAF.FRMA.Algoritmo = "SHA1withRSA"

        doc.TED.DD.TSTED = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)

        doc.TED.FRMT = New DocumentoDTE.SiiDte.FRMTA()
        doc.TED.FRMT.Algoritmo = DocumentoDTE.EnvioDTE_v10Lib.Enumerations.FRMTA_Algoritmo.SHA1withRSA

        doc.TmstFirma = New LiquidTechnologies.Runtime.Net20.XmlDateTime(2010, 1, 1, 0, 0, 0)
    End Sub


End Module
